using System;

namespace MyIntegrator
{
	
	public class MyIntegrator
	{
		public const int MsIntRsltSize = 250;
		public const byte MsIntDfilterOff = 0;
		public const byte MsInt5Point = 1;
		public const byte MsInt7Point = 2;
		public const byte MsInt9Point = 3;
		public const byte MsIntTanskim = 1;
		public const byte MsIntCentroid = 1;
		public const int MsIntMaxSmooth = 9;
		public const int MsIntRekeyval = 1000000;
		public const int Minus4 = 0;
		public const int Minus3 = 1;
		public const int Minus2 = 2;
		public const int Minus1 = 3;
		public const int Zero = 4;
		public const int Plus1 = 5;
		public const int Plus2 = 6;
		public const int Plus3 = 7;
		public const int Plus4 = 8;
		public const int Plus5 = 9;
		public const int Numahead = 5;
		public const int NoError = 0;
		public const int IdsMsIntTooManyPeaks = 1;
		public const int IdsMsIntBufOverflow = 2;
		public const int IdsMsIntZerodiv = 3;
		public Peak[] Peaks;
		public double[] Coeff5Pt;
		public double[] Coeff7Pt;
		public double[] Coeff9Pt;
		public int PeakCount=0;
		Signal _signal;
		Settings _settings;
		public MyIntegrator(Signal signal, Settings settings)
		{
			_settings = settings;
			_signal = signal;
			Peaks = new Peak[250];
			for (var i=0; i<Peaks.Length;i++)
			{
				Peaks[i] = new Peak();
			}
			Coeff5Pt = new double[]
			{
				0.083333333333333329,
				-0.66666666666666663,
				0.0,
				0.66666666666666663,
				-0.083333333333333329
			};
			Coeff7Pt = new double[]
			{
				0.0873015873015873,
				-0.26587301587301587,
				-0.23015873015873015,
				0.0,
				0.23015873015873015,
				0.26587301587301587,
				-0.0873015873015873
			};
			Coeff9Pt = new double[]
			{
				0.0723905723905724,
				-0.11952861952861953,
				-0.16245791245791247,
				-0.10606060606060606,
				0.0,
				0.10606060606060606,
				0.16245791245791247,
				0.11952861952861953,
				-0.0723905723905724
			};
		}

		public double Deriv(int points, double[] dArr, int iStartOffset)
		{
			var result = 0.0;
			switch (points)
			{
			case 7:
				for (var i = 0; i < points; i += 1)
				{
					var factor = Coeff7Pt[i];
					result += factor * dArr[i + iStartOffset];
				}
				return result;
			case 9:
				for (var i = 0; i < points; i += 1)
				{
					var factor = Coeff9Pt[i];
					result += factor * dArr[i + iStartOffset];
				}
				return result;
			}
			for (var i = 0; i < points; i += 1)
			{
				var factor = Coeff5Pt[i];
				result += factor * dArr[i + iStartOffset];
			}
			return result;
		}
		double TrapArea(int i, int ioff, double valley, bool switchBl)
		{
			double result;
			var data = _signal.y[Peaks[i-1].xLeft - ioff + 1];
			var data2 = _signal.y[Peaks[i-1].xRight - ioff + 1];
			var data3 = _signal.y[Peaks[i-1].xTop - ioff + 1];
			var num = Peaks[i-1].xRight - Peaks[i-1].xLeft;
			var num2 = Math.Min(data, data2);
			var num3 = Math.Abs(data - data2);
			var num4 = data + data2;
			var num5 = data3 - num2;
			if (num5 == 0.0)
			{
				result = 0.0;
				return result;
			}
			var num6 = num3 / num5;
			var num7 = num2 * num;
			var num8 = num4 * num / 2.0;
			if (switchBl)
			{
				var num9 = valley;
				valley = num6;
				num6 = num9;
			}
			result = Peaks[i-1].areaRaw - (int)MyMath.Round((num6 < valley) ? num7 : num8);
			Peaks[i-1].baselineLeft = data;
			Peaks[i-1].baselineRight = data2;
			Peaks[i-1].peakHeight = data3;
			return result;
		}
		int MsIntIntegrate(bool fNewThreshMeth, int start, int stop, int ioff, double rThresh, double rArea, int bunch, int maxpk, double eThresh, double valley, double scsmallamt, int filPoints, bool switchBl)
		{
			var array = new double[10];
			var array2 = new double[10];
			var num = 0.0;
			var num2 = 0.0;
			bool flag2;
			var flag = flag2 = false;
			var num3 = (bunch != 0) ? Math.Abs(bunch) : 1;
			array[2] = 0.0;
			array[3] = _signal.y[start];
			array[0] = (array[1] = array[2]);
			array[4] = _signal.y[start + num3];
			array[5] = _signal.y[start + 2 * num3];
			array[6] = _signal.y[start + 3 * num3];
			array[7] = _signal.y[start + 4 * num3];
			array[8] = _signal.y[start + 5 * num3];
			array[9] = _signal.y[start + 6 * num3];
			var num4 = rThresh / 2.0;
			var num5 = eThresh / 2.0;
			var num6 = 0.0;
			var sMax = ioff;
			var num7 = 0;
			var num8 = start + 2 * num3;
			int result;
			while (num8 < stop - 5L * num3)
			{
				var a = num8 + 5 * num3;
				var num9 = num8 + ioff - 1;
				var num10 = _signal.y[num8];
				num2 += num10;
				if (num10 > num6)
				{
					num6 = num10;
					sMax = num9;
				}
				while (num8 % num3 == 0)
				{
					array[0] = array[1];
					array[1] = array[2];
					array[2] = array[3];
					array[3] = array[4];
					array[4] = array[5];
					array[5] = array[6];
					array[6] = array[7];
					array[7] = array[8];
					array[8] = array[9];
					array[9] = _signal.y[a];
					var num11 = num;
					if (filPoints != 5 && filPoints != 7 && filPoints != 9)
					{
						filPoints = 5;
					}
					if (bunch < 0 && filPoints == 5)
					{
						array2[0] = array[3];
						array2[1] = (array[3] + array[4]) / 2.0;
						array2[2] = array[5];
						array2[3] = (array[5] + array[6] + array[7]) / 3.0;
						array2[4] = array[7];
						num = Deriv(filPoints, array2, 0);
					}
					else
					{
						num = Deriv(filPoints, array, (5 - (filPoints - 1) / 2));
					}
					var num12 = array[4];
					var num13 = num4 * (num12 - scsmallamt);
					var nw1=false;
					if (!flag2)
					{
						if (num11 > num13)
						{
							if (num7 >= maxpk)
							{
								result = 1;
								PeakCount = num7;
								return result;
							}
							num7++;
							flag2 = true;
							sMax = num9;
							num6 = array[4];
							num10 = array[4];
							if (array[3] < array[4])
							{
								Peaks[num7 - 1].areaRaw = 0.0;
								Peaks[num7 - 1].xLeft = num9 - num3;
							} else {
								Peaks[num7 - 1].areaRaw = (int)Math.Round(-num2);
								Peaks[num7 - 1].xLeft = num9;
							}
							nw1 = true;
						}
						if (!nw1)
						{
							break;
						}
					}
					Peaks[ num7 - 1].areaRaw = Peaks[ num7 - 1].areaRaw + (int)MyMath.Round(num2);
					if (!flag)
					{
						if (num > 0.0)
						{
							break;
						}
						flag = true;
					}
					if (num8 != (stop - 1) - 5L * num3 && (fNewThreshMeth ? (num < -(num5 * (array[5] - scsmallamt))) : (num < -num13 * eThresh / rThresh)))
					{
						break;
					}
					if (array[5] < array[4])
					{
						Peaks[num7 - 1].xRight = num9 + num3;
						var num14 = 1;
						while (num14 <= num3)
						{
							var peak = num7 - 1;
							Peaks[peak].areaRaw = Peaks[peak].areaRaw + _signal.y[num9 + num14];
							num14++;
						}
					}
					else
					{
						Peaks[num7 - 1].xRight = num9;
					}
					Peaks[num7 - 1].xTop = sMax;
					Peaks[num7 - 1].areaCorrected = TrapArea(num7, ioff, valley, switchBl);
					flag2 = false;
					flag = false;
					num6 = 0.0;
					if (Peaks[num7 - 1].areaCorrected <= rArea || Peaks[num7 - 1].xTop == Peaks[num7 - 1].xRight)
					{
						Peaks[num7].clear();
						num7--;
						break;
					}
					break;
				}
				num2 = 0.0;
				num8++;
			}
			if (flag2)
			{
				Peaks[num7].clear();
				num7--;
			}
			result = 0;
			PeakCount = num7;
			return result;
		}
		int MsIntLbase(int j, int blScan)
		{
			if (j != 1)
			{
				for (var num = j; num >= 2; num -= 1)
				{
					var num2 = num - 1;
					if ((Peaks[num - 1].xLeft - Peaks[num2 - 1].xRight) > blScan - 1)
					{
						return num-1;
					}
				}
			}
			return 0;
		}
		int MsIntRbase(int j, int blScan)
		{
			if (j != PeakCount)
			{
				for (var num = j; num < PeakCount; num += 1)
				{
					var num2 = num + 1;
					if ((Peaks[num2 - 1].xLeft - Peaks[num - 1].xRight) >= blScan - 1)
					{
						return num-1;
					}
				}
			}
			return PeakCount-1;
		}
		int MsIntNewB(int blScan, int ioff, double valley, int bunch, bool switchBl)
		{
			var y = 0.0;
			var y2 = 0.0;
			var linReg = new Linear();
			var num = 0;
			var num2 = 0;
			int p;
			for (var num3 = 1; num3 <= PeakCount; num3 += 1)
			{
				p = MsIntLbase(num3, blScan);
				num = Peaks[p].xLeft;
				y = Peaks[p].baselineLeft;
				
				p = MsIntRbase(num3, blScan);
				num2 = Peaks[p].xRight;
				y2 = Peaks[p].baselineRight;
				
				linReg.slpin8(num, y, num2, y2);
				var sTrap = Peaks[num3 - 1].baselineLeft;
				var sBlD = Peaks[num3 - 1].baselineRight;
				var sPkht = Peaks[num3 - 1].peakHeight;
				var num4 = sPkht - Math.Min(sTrap, sBlD);
				if (num4 == 0.0)
				{
					return 3;
				}
				var num5 = Math.Abs(sTrap - sBlD) / num4;
				var sStart = Peaks[num3 - 1].xLeft;
				var val = sTrap;
				var val2 = linReg.msIntFitLn8(sStart);
				var sStop = Peaks[num3 - 1].xRight;
				var val3 = sBlD;
				var val4 = linReg.msIntFitLn8(sStop);
				if (switchBl)
				{
					var num6 = valley;
					valley = num5;
					num5 = num6;
				}
				char lType;
				char rType;
				if (num5 < valley)
				{
					Peaks[num3 - 1].baselineLeft = (int)MyMath.Round(Math.Min(val, Math.Min(val3, val2)));
					Peaks[num3 - 1].baselineRight = (int)MyMath.Round(Math.Min(val, Math.Min(val3, val4)));
					y = Peaks[num3 - 1].baselineLeft;
					y2 = Peaks[num3 - 1].baselineRight;
					lType = 'V';
					rType = 'V';
					if (sTrap == Peaks[num3 - 1].baselineLeft)
					{
						lType = 'B';
					}
					if (sBlD == Peaks[num3 - 1].baselineRight)
					{
						rType = 'B';
					}
					linReg.slpin8(num, y, num2, y2);
				}
				else
				{
					Peaks[num3 - 1].baselineLeft = sTrap;
					Peaks[num3 - 1].baselineRight = sBlD;
					lType = 'B';
					rType = 'B';
					Peaks[num3 - 1].msIntSetTangent(ioff, linReg, _signal);
				}
				if (switchBl)
				{
					valley = num5;
				}
				var sMax = Peaks[num3 - 1].xTop;
				var num7 = linReg.msIntFitLn8(sMax);
				Peaks[num3 - 1].peakHeight = (int)MyMath.Round(Peaks[num3 - 1].peakHeight - num7);
				Peaks[num3 - 1].areaCorrected = Peaks[num3 - 1].areaRaw - (Peaks[num3 - 1].baselineLeft + Peaks[num3 - 1].baselineRight) * (Peaks[num3 - 1].xRight - Peaks[num3 - 1].xLeft) / 2.0;
				Peaks[num3 - 1].typeLeft = lType;
				Peaks[num3 - 1].typeRight = rType;
			}
			return 0;
		}
		public int FindBasePeak()
		{
			var num = 0.0;
			var pMaxIndex = 0;
			for (var num2 = 0; num2 < PeakCount; num2 += 1)
			{
				if (Peaks[num2].areaCorrected > num)
				{
					num = Peaks[num2].areaCorrected;
					pMaxIndex = (num2 + 1);
				}
			}
			return pMaxIndex;
		}
		public bool CompareByCorrArea(Peak a, Peak b)
		{
			return a.areaCorrected >= b.areaCorrected;
		}
		public bool CompareByRt(Peak a, Peak b)
		{
			return a.xTop < b.xTop;
		}
		public void Sift(Peak pSortTempRec, int ulLeft, int ulRight, int iSortCompareFunction)
		{
			var num = ulLeft;
			var num2 = num << 1;
			pSortTempRec = Peaks[num - 1];
			while (num2 <= ulRight)
			{
				if (num2 < ulRight)
				{
					if (1 == iSortCompareFunction)
					{
						if (CompareByCorrArea(Peaks[num2 - 1], Peaks[num2]))
						{
							num2++;
						}
					}
					else
					{
						if (2 == iSortCompareFunction && CompareByRt(Peaks[num2 - 1], Peaks[num2]))
						{
							num2++;
						}
					}
				}
				if (1 == iSortCompareFunction)
				{
					if (!CompareByCorrArea(pSortTempRec, Peaks[num2 - 1]))
					{
						break;
					}
				}
				else
				{
					if (2 == iSortCompareFunction && !CompareByRt(pSortTempRec, Peaks[num2 - 1]))
					{
						break;
					}
				}
				Peaks[(num - 1)] = Peaks[num2 - 1];
				num = num2;
				num2 = num << 1;
			}
			Peaks[(num - 1)] = pSortTempRec;
		}
		public void SortHeapSort(int iCompareFunction)
		{
			var peak = new Peak();
			var num = (PeakCount >> 1) + 1;
			var num2 = PeakCount;
			while (num > 1)
			{
				num--;
				Sift(peak, num, num2, iCompareFunction);
			}
			while (num2 > 1)
			{
				peak = Peaks[0];
				Peaks[0] = Peaks[num2 - 1];
				Peaks[num2 - 1] = peak;
				num2--;
				Sift(peak, num, num2, iCompareFunction);
			}
		}
		public int Integrate()
		{
			double sclargstamt=0;
			double scsmallamt=0;
			double rtHigh=0;
			var rtLow=0.01;
			var dataStart = 1;
			var dataLength = _signal.GetSize();
			var centroidFunction = 1;
			var filPoints = 0;
			var num = 0;
			var threshold = (_settings.isThresholdPercent) ? _settings.threshold : (-_settings.threshold);
			var maxPeaks = _settings.maxPeaks;
			var baselineResetPoints = _settings.baselineResetPoints;
			var bunch = _settings.bunch;
			var leftThreshold = _settings.leftThreshold;
			var rightThreshold = _settings.rightThreshold;
			var valleyPercent = _settings.valleyPercent;
			var num6 = maxPeaks;
			var blScan = (baselineResetPoints + 1);

			if (baselineResetPoints == -1)
			{
				blScan = 1;
			}
			if (baselineResetPoints == 0)
			{
				blScan = 6;
			}
			var num7 = bunch;
			if (num7 == 0)
			{
				num7 = 1;
			}
			if (valleyPercent < 1.0)
			{
				valleyPercent = 100.0;
			}
			valleyPercent /= 100.0;
			rightThreshold = -rightThreshold;
			switch (_settings.smoothPoints)
			{
			case 0:
				filPoints = 0;
				break;
			case 1:
				filPoints = 5;
				break;
			case 2:
				filPoints = 7;
				break;
			case 3:
				filPoints = 9;
				break;
			}
			var switchBl = _settings.isTangentElseDrop;
			var start = 0;
			var num8 = 0;
			double num9;
			if (threshold >= 0.0)
			{
				num9 = threshold * (sclargstamt - scsmallamt) / 100.0;
			}
			else
			{
				num9 = -threshold;
			}
			while (dataLength != 1 && rtHigh != rtLow && dataLength >= (Math.Abs(num7) * 6 + 1))
			{
				var rArea = num9 * 0.95 / ((rtHigh - rtLow) * 60.0 / (dataLength - 1));
				var num10 = MsIntIntegrate(false, start, dataLength, dataStart, leftThreshold, rArea, num7, _settings.maxPeaks, rightThreshold, valleyPercent, scsmallamt, filPoints, switchBl);
				if (num10 != 0 && num10 != 1)
				{
					return num10;
				}
				if (num10 == 1)
				{
					leftThreshold *= 2.0;
					rightThreshold *= 2.0;
					num9 *= 2.0;
					num8++;
					if (num8 < 9)
					{
						continue;
					}
				}
				if (PeakCount == 0)
				{
					break;
				}
				if (num10 == 1)
				{
					return 2;
				}
				if ((num10 = MsIntNewB(blScan, dataStart, valleyPercent, num7, switchBl)) != 0)
				{
					return num10;
				}
				num = FindBasePeak();
				var num12 = Peaks[num].areaCorrected;				
				
				var num13 = num12 / 100.0;
				if (num > 0)
				{
					double data = _signal.x[Peaks[num - 1].xLeft - dataStart + 1];
					double data2 = _signal.x[Peaks[num - 1].xRight - dataStart + 1];
					var num14 = (_settings.scaleFactor == 0.0) ? 1.0 : (_settings.scaleFactor * (data2 - data) / 1000.0 / (Peaks[num - 1].xRight - Peaks[num - 1].xLeft));
					num13 *= num14;
				}
				int num15;
				for (num15 = 0; num15 < PeakCount; num15 += 1)
				{
					double data = _signal.x[Peaks[num15].xLeft - dataStart + 1];
					double data2 = _signal.x[Peaks[num15].xRight - dataStart + 1];
					if (Peaks[num15].xRight == Peaks[num15].xLeft)
					{
						return 3;
					}
					var num14 = (_settings.scaleFactor == 0.0) ? 1.0 : (_settings.scaleFactor * (data2 - data) / 1000.0 / (Peaks[num15].xRight - Peaks[num15].xLeft));
					Peaks[num15].areaRaw = (int)MyMath.Round(Peaks[num15].areaRaw * num14);
					Peaks[num15].areaCorrected = (int)MyMath.Round(Peaks[num15].areaCorrected * num14);
				}
				SortHeapSort(1);
				num15 = PeakCount + 1;
				if (threshold > 0.0)
				{
					var basePeak = FindBasePeak();
					var area = Peaks[basePeak].areaCorrected / 100.0;				
					
					var num17 = Math.Max(area, num13);
					if (threshold >= 100.0)
					{
						threshold = 99.999;
					}
					if (num13 != 0.0)
					{
						for (num15 = 1; num15 <= PeakCount; num15 += 1)
						{
							if (Peaks[num15 - 1].areaCorrected / num17 < threshold)
							{
								break;
							}
						}
					}
				}
				else
				{
					num15 = 1;
					while (num15 <= PeakCount && Peaks[num15 - 1].areaCorrected >= Math.Abs(threshold))
					{
						num15++;
					}
				}
				PeakCount = num15 - 1;
				if (PeakCount > num6)
				{
					PeakCount = num6;
				}
				for (num15 = 0; num15 < _settings.maxPeaks; num15 += 1)
				{
					Peaks[num15].xLeft = Peaks[num15].xLeft + num15 * 1000000;
				}
				SortHeapSort(2);
				CalcPeakRetentionTimes(_settings.isCentroid, centroidFunction);
				return 0;
			}
			PeakCount = 0;
			return 0;
		}
		void CalcPeakRetentionTimes(bool isCentroid, int centroidFunction)
		{
			for(var i=0;i<PeakCount;i++)
			{
				Peaks[i].calcPeakRetentionTime(_signal, isCentroid, centroidFunction);
			}
		}
		void ManualIntegrate(int timeLo, int timeHi, int lYBaselineLeft, int lYBaselineRight, int ytype, bool checkBaseline, int centroidFunction)
		{
			var num = _signal.GetSize();
			int i;
			for (i = 0; i < num; i++)
			{
				double num2 = _signal.x[i];
				if (num2 >= timeLo)
				{
					break;
				}
			}
			if (i == num)
			{
				i--;
			}
			if (i < 0)
			{
				i = 0;
			}
			var num3 = SelectDataXVal(i, timeLo, lYBaselineLeft);
			while (i < num)
			{
				double num2 = (_signal.x[i]);
				if (num2 >= timeHi)
				{
					break;
				}
				i++;
			}
			if (i == num)
			{
				i--;
			}
			if (i < 0)
			{
				i = 0;
			}
			var num4 = SelectDataXVal(i, timeHi, lYBaselineRight);
			var num5 = _signal.y[num3];
			var num6 = _signal.y[num4];
			if ((num5 < lYBaselineLeft || num6 < lYBaselineRight) && checkBaseline)
			{
				throw new ArgumentOutOfRangeException("Illegal Manual Integration");
			}
			Peaks[0].xLeft = num3;
			Peaks[0].xRight = num4;
			Peaks[0].xTop = num3;
			num5 = (_signal.y[num3]);
			num6 = (_signal.y[num4]);
			Peaks[0].areaRaw = -num5;
			var num7 = 0.0;
			for (i = num3; i <= num4; i++)
			{
				double num2 = i;
				var data = _signal.y[i];
				if (data > num7)
				{
					num7 = data;
					Peaks[0].xTop = (int)MyMath.Round(num2);
				}
				Peaks[0].areaRaw = Peaks[0].areaRaw + data;
			}
			var num8 = Math.Min(num5, num6);
			var num9 = Peaks[0].xRight - Peaks[0].xLeft;
			Peaks[0].baselineLeft = ((ytype != 0) ? (lYBaselineLeft) : num8);
			Peaks[0].baselineRight = ((ytype != 0) ? (lYBaselineRight) : num8);
			Peaks[0].areaCorrected = Peaks[0].areaRaw - (Peaks[0].baselineLeft + Peaks[0].baselineRight) / 2.0 * num9;
			double rtLeft = (_signal.x[Peaks[0].xLeft]);
			double rtRight = (_signal.x[Peaks[0].xRight]);
			var rtWidth = rtRight - rtLeft;
			var width = Peaks[0].xRight - Peaks[0].xLeft;
			double num14;
			if (width == 0)
			{
				num14 = 0.0;
			}
			else
			{
				num14 = ((_settings.scaleFactor == 0.0) ? 1.0 : (_settings.scaleFactor * rtWidth / 1000.0 / width));
			}
			Peaks[0].areaRaw = (int)MyMath.Round(Peaks[0].areaRaw * num14);
			Peaks[0].areaCorrected = (int)MyMath.Round(Peaks[0].areaCorrected * num14);
			double num15 = (_signal.x[Peaks[0].xTop]);
			Peaks[0].peakHeight = num7 - ((rtWidth == 0.0) ? 0 : (int)MyMath.Round(lYBaselineLeft + (num15 - rtLeft) / rtWidth * (lYBaselineRight - lYBaselineLeft)));
			Peaks[0].calcPeakRetentionTime(_signal, _settings.isCentroid, centroidFunction);
		}
		int SelectDataXVal(int i, int x, int y)
		{
			if (i == 0)
			{
				return 0;
			}
			double x0 = _signal.x[i - 1];
			var y0 = _signal.y[i - 1];
			double x1 = _signal.x[i];
			var y1 = _signal.y[i];
			if (x1 - x > x - x0)
			{
				if (y0 < y)
				{
					return i;
				}
			}
			else
			{
				if (y1 >= y)
				{
					return i;
				}
			}
			return i--;
		}
	}
}

