using System;
namespace MyIntegrator
{
	public class Peak
	{
		public int xLeft;
		public int xTop;
		public int xRight;
		public double areaRaw;
		public double areaCorrected;
		public double baselineLeft;
		public double baselineRight;
		public double peakHeight;
		public char typeLeft;
		public char typeRight;
		public int scanLeft;
		public int scanTop;
		public int scanRight;
		public Peak()
		{
			clear();
		}
		public void clear()
		{
			xLeft = 0;
			xTop = 0;
			xRight = 0;
			areaRaw = 0.0;
			areaCorrected = 0.0;
			baselineLeft = 0.0;
			baselineRight = 0.0;
			peakHeight = 0.0;
			typeLeft = '\0';
			typeRight = '\0';
			scanLeft = 0;
			scanTop = 0;
			scanRight = 0;
		}
		int centroidAtHeight(Signal signal)
		{
			var topHeight = signal.y[scanTop];
			var baselineLow = Math.Min(baselineLeft, baselineRight);
			var maxHeight = topHeight;
			if (baselineLeft > maxHeight)
			{
				maxHeight = baselineLeft;
			}
			if (baselineRight > maxHeight)
			{
				maxHeight = baselineRight;
			}
			var centroid = 0;
			var sh = 0.0;
			var sa = 0.0;
			for (var scan = scanLeft; scan <= scanRight; scan++)
			{
				var y = signal.y[scan];
				if (y > maxHeight)
				{
					var h = y - maxHeight;
					sh += h;
					sa += h * signal.x[scan];
					if (sh != 0.0)
					{
						centroid = (int)MyMath.Round(sa / sh);
					}
				}
			}
			return centroid;
		}
		public void calcPeakRetentionTime(Signal signal, bool findCentroid, int centroidFunction)
		{
			xLeft = xLeft % 1000000;
			scanLeft = xLeft;
			xLeft = signal.x[scanLeft];
			scanTop = xTop;
			xTop = signal.x[scanTop];
			scanRight = xRight;
			xRight = signal.x[scanRight];
			if (findCentroid)
			{
				if (1 == centroidFunction)
				{
					xTop = peakCentroid(signal);
				}
				else
				{
					if (2 == centroidFunction)
					{
						xTop = centroidAtHeight(signal);
					}
				}
			}
		}
		int peakCentroid(Signal signal)
		{
			double xWidth = (xRight - xLeft);
			var baselineDelta = baselineRight - baselineLeft;
			var linear = new Linear();
			if (xWidth != 0.0)
			{
				linear.slope = baselineDelta / xWidth;
				linear.interc = baselineLeft - linear.slope * xLeft;
			}
			else
			{
				linear.slope = 1.0;
				linear.interc = 0.0;
			}
			var centroid = 0;
			double sh;
			var sa = sh = 0.0;
			for (var scan = scanLeft; scan <= scanRight; scan++)
			{
				double x = signal.x[scan];
				var h = signal.y[scan] - linear.msIntFitLn8(x);
				sh += h;
				sa += h * x;
			}
			if (sh != 0.0)
			{
				centroid = (int)MyMath.Round(sa / sh);
			}
			return centroid;
		}
		public void msIntSetTangent(int ioff, Linear linear, Signal signal)
		{
			var s_start = xLeft;
			var s_max = xTop;
			var s_stop = xRight;
			var baselineLeft = this.baselineLeft;
			var baselineRight = this.baselineRight;
			int num3;
			int num4;
			bool flag;
			if (baselineLeft > baselineRight)
			{
				num3 = s_stop - 1;
				num4 = -1;
				flag = true;
			}
			else
			{
				num3 = s_start + 1;
				num4 = 1;
				flag = false;
			}
			var cont = true;
			while (cont)
			{
				cont = false;
				linear.slpin8(s_start, baselineLeft, s_stop, baselineRight);
				var num5 = num3;
				while ((num4 == 1) ? (num5 <= s_max - num4) : (num5 >= s_max - num4))
				{
					var num6 = num5;
					var num7 = linear.msIntFitLn8(num6);
					var num8 = num5 - ioff + 1;
					var data = signal.y[num8];
					if (num7 > data)
					{
						num5 += num4;
						if (flag)
						{
							data = signal.y[xRight];
							xRight = xRight - 1;
							this.baselineRight = signal.y[xRight];
							s_stop = xRight;
							baselineRight = this.baselineRight;
							num3--;
						}
						else
						{
							data = signal.y[xLeft];
							xLeft = xLeft + 1;
							this.baselineLeft = signal.y[xLeft];
							s_start = xLeft;
							baselineLeft = this.baselineLeft;
							num3++;
						}
						areaRaw = areaRaw - data;
						cont = true;
						break;
					} else {
						num5 += num4;
					}
				}
			}
		}
	}
}
