﻿using System;

namespace QuantReviewTest.Chrom
{
    internal class SignalSmoother
    {
        private readonly int _kernelSize;
        private readonly double _sigma;
        readonly double[] _kernel;

        public SignalSmoother(int kernelSize, double sigma)
        {
            _kernelSize = kernelSize;
            _sigma = sigma;
            _kernel = Gaussian1D(_sigma, _kernelSize);
        }

        public double[] Smooth(double[] data)
        {
            var outputData = new double[data.Length];

            for (var i = 0; i < data.Length; ++i)
            {
                var sum = 0.0;

                for (var j = 0; j < _kernelSize; ++j)
                {
                    var k = i - _kernelSize / 2 + j;

                    // Check that the convolution is within the boundaries of the input data
                    if (k >= 0 && k < data.Length)
                    {
                        sum += data[k] * _kernel[j];
                    }
                }

                outputData[i] = sum;
            }

            return outputData;
        }

        public static double[] Gaussian1D(double sigma, int size)
        {
            var kernel = new double[size];
            var sum = 0.0;
            var halfSize = size / 2.0;

            for (var i = 0; i < size; ++i)
            {
                var x = i - halfSize;
                kernel[i] = Math.Exp(-(x * x) / (2.0 * sigma * sigma)) / (Math.Sqrt(2.0 * Math.PI) * sigma);
                sum += kernel[i];
            }

            // Normalize the kernel so that its elements sum to 1
            for (var i = 0; i < size; ++i)
            {
                kernel[i] /= sum;
            }

            return kernel;
        }
    }
}
