﻿using System;
using System.Drawing;
using System.Linq;
using FarPoint.Win.Spread;
using FarPoint.Win.Spread.Visualization;
using GrapeCity.CalcEngine;
using GrapeCity.Spreadsheet;
using MyIntegrator;
using QuantReviewTest.Chrom;
using Color = System.Drawing.Color;
using ValueType = GrapeCity.CalcEngine.ValueType;

namespace QuantReviewTest.Functions
{
    public class ChromVisualFunction
    {
        private static DataSetContext _dataSetContext;

        public static void Register(FpSpread spread, DataSetContext dataSetContext)
        {
            _dataSetContext = dataSetContext;
            spread.AddCustomFunction(new Chrom(true));
            spread.AddCustomFunction(new Chrom(false));
        }

        public class Chrom : VisualFunction
        {
            private readonly bool _asText;
            public short[,] Arg1Array;
            public short[,] Arg2Array;
            public IEvaluationContext EvaluationContext;

            public Chrom(bool asText) : base(asText?"ChromText":"Chrom", 2, 2, FunctionAttributes.Visual | FunctionAttributes.Array | FunctionAttributes.Volatile, new ChromFunctionVisualizer())
            {
                _asText = asText;
            }

            protected override bool Evaluate(IArguments arguments, IValue result)
            {
                try
                {
                    Evaluate2(arguments, result);
                }
                catch (Exception e)
                {
                    Console.WriteLine(e);
                }
                return _asText;
            }

            protected void Evaluate2(IArguments arguments, IValue result)
            {
                EvaluationContext = arguments.EvaluationContext;
                Arg1Array = GetArgIdArray(result, arguments, 0);
                Arg2Array = GetArgIdArray(result, arguments, 1);

                var rowCount1 = Arg1Array.GetLength(0);
                var colCount1 = Arg1Array.GetLength(1);

                var rowCount2 = Arg2Array.GetLength(0);
                var colCount2 = Arg2Array.GetLength(1);

                if (rowCount1 * colCount1 * rowCount2 * colCount2 == 1)
                {
                    var compound = _dataSetContext.GetCompound(Arg1Array[0, 0], Arg2Array[0, 0]);
                    SetDataValue(result, compound);
                    return;
                }


                if (rowCount2 * colCount2 == 1)
                {
                    var resArray = result.CreateValuesArray(rowCount1, colCount1);
                    for (var r = 0; r < Arg1Array.GetLength(0); r++)
                    {
                        for (var c = 0; c < Arg1Array.GetLength(1); c++)
                        {
                            var compound = _dataSetContext.GetCompound(Arg1Array[r, c], Arg2Array[0, 0]);
                            SetDataValue(resArray[r, c], compound);
                        }
                    }
                    result.SetValue(resArray);
                }

                if (rowCount1 * colCount1 == 1)
                {
                    var resArray = result.CreateValuesArray(rowCount2, colCount2);
                    for (var r = 0; r < Arg2Array.GetLength(0); r++)
                    {
                        for (var c = 0; c < Arg2Array.GetLength(1); c++)
                        {
                            var compound = _dataSetContext.GetCompound(Arg2Array[r, c], Arg1Array[0, 0]);
                            SetDataValue(resArray[r, c], compound);
                        }
                    }
                    result.SetValue(resArray);
                }

                if (rowCount2 * colCount1 == 1)
                {
                    var resArray = result.CreateValuesArray(rowCount1, colCount2);
                    for (var r = 0; r < Arg1Array.GetLength(0); r++)
                    {
                        for (var c = 0; c < Arg2Array.GetLength(1); c++)
                        {
                            var compound = _dataSetContext.GetCompound(Arg1Array[r, 0], Arg2Array[0, c]);
                            SetDataValue(resArray[r, c], compound);
                        }
                    }
                    result.SetValue(resArray);
                }

                if (rowCount1 * colCount2 == 1)
                {
                    var resArray = result.CreateValuesArray(rowCount2, colCount1);
                    for (var r = 0; r < Arg2Array.GetLength(0); r++)
                    {
                        for (var c = 0; c < Arg1Array.GetLength(1); c++)
                        {
                            var compound = _dataSetContext.GetCompound(Arg2Array[r, 0], Arg1Array[0, c]);
                            SetDataValue(resArray[r, c], compound);
                        }
                    }
                    result.SetValue(resArray);
                }

            }

            private void SetDataValue(IPrimitiveValue result, Compound compound)
            {
                result.SetValue($"{compound.SampleId* 10000 + compound.CompoundId}");
            }

            private short[,] GetArgIdArray(IValue result, IArguments arguments, int pIndex)
            {
                var argument = arguments[pIndex];
                short[,] res;
                switch (argument.ValueType)
                {
                    case ValueType.AdjustableReference:
                    case ValueType.Reference3D:
                    case ValueType.Empty:
                    case ValueType.Error:
                    case ValueType.Logical:
                    case ValueType.Text:
                    case ValueType.DateTime:
                    case ValueType.TimeSpan:
                    case ValueType.Integer:
                    case ValueType.Image:
                    case ValueType.Delegate:
                    case ValueType.Object:
                    case ValueType.ArrayReference:
                        break;
                    case ValueType.Number:
                        {
                            res = new short[1, 1];
                            res[0, 0] = Convert.ToInt16(argument.GetValue());
                            return res;
                        }
                    case ValueType.ValuesArray:
                        {
                            var argArray = argument.GetValuesArray();
                            res = new short[argArray.RowCount, argArray.ColumnCount];
                            for (var r = 0; r < res.GetLength(0); r++)
                                for (var c = 0; c < res.GetLength(1); c++)
                                    res[r, c] = Convert.ToInt16(argArray[r, c].GetNumber());
                            return res;
                        }
                    case ValueType.Reference:
                    case ValueType.MultiReference:
                        {
                            var reference = argument.GetReference(arguments.EvaluationContext);
                            var rowCount = reference.Row2 - reference.Row + 1;
                            var colCount = reference.Column2 - reference.Column + 1;
                            var valuesArray = result.CreateValuesArray(rowCount, colCount);
                            arguments.EvaluationContext.ReferenceSource.GetValue(arguments.EvaluationContext, reference.Row, reference.Column, reference.Row2, reference.Column2, valuesArray, true);
                            res = new short[rowCount, colCount];

                            for (var r = 0; r < res.GetLength(0); r++)
                                for (var c = 0; c < res.GetLength(1); c++)
                                    res[r, c] = Convert.ToInt16(valuesArray[r, c]?.GetNumber());
                            return res;
                        }
                }
                return null;
            }
        }

        public class ChromFunctionVisualizer : FunctionVisualizer
        {
            private static readonly Pen PenSignal = new Pen(Color.Black, 2);
            private static readonly Pen PenIntegration = new Pen(Color.Red, 2);
            private static readonly SignalSmoother SignalSmoother = new SignalSmoother(41, 5);
            private static readonly MyIntegrator.MyIntegrator Integrator;

            protected override void PaintCell(
                Graphics graphics,
                Rectangle rect,
                object cellValue,
                ref StyleFormat styleFormat,
                IWorksheet worksheet)
            {

                //HOW TO get the parameters here, like in Evaluate

                rect.Width--;
                rect.Height--;

                if (cellValue is VisualizationData visualizationData)
                {
                    var chromGenerator = new ChromGenerator(rect.Width - 4, rect.Height - 4, 5);
                    var data = chromGenerator.Generate();
                    data = SignalSmoother.Smooth(data);

                    var max = data.Max();
                    var factor = (float)((rect.Height - 4) / max);
                    for (var i = 0; i < data.Length; i++)
                        data[i] *= factor;


                    var signal = new Signal(0)
                    {
                        x = data.Select((_, i) => i).ToArray(),
                        y = data.Select(p => p).ToArray()
                    };

                    var settings = new Settings();
                    var integrator = new MyIntegrator.MyIntegrator(signal, settings);
                    integrator.Integrate();
                    var peaks = integrator.Peaks.Where(p => p.scanTop > 0).ToList();
                    var points = data.Select((d, i) => new PointF(rect.Left + 2 + i, rect.Bottom - 2 - (float)d)).ToArray();
                    graphics.DrawLines(PenSignal, points);


                    foreach (var peak in peaks)
                    {
                        var integrationPoints = new PointF[4];

                        integrationPoints[0].X = rect.Left + 2 + (float)peak.xLeft;
                        integrationPoints[0].Y = rect.Bottom - 2 - (float)signal.y[peak.scanLeft];

                        integrationPoints[1].X = rect.Left + 2 + (float)peak.xLeft;
                        integrationPoints[1].Y = rect.Bottom - 2 - (float)peak.baselineLeft;

                        integrationPoints[2].X = rect.Left + 2 + (float)peak.xRight;
                        integrationPoints[2].Y = rect.Bottom - 2 - (float)peak.baselineRight;

                        integrationPoints[3].X = rect.Left + 2 + (float)peak.xRight;
                        integrationPoints[3].Y = rect.Bottom - 2 - (float)signal.y[peak.scanRight];

                        graphics.DrawLines(PenIntegration, integrationPoints);
                    }
                }
            }
        }
    }
}