﻿using System;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using FarPoint.Excel;
using FarPoint.Win.Spread;
using FarPoint.Win.Spread.DrawingSpace;
using GrapeCity.Spreadsheet;
using QuantReviewTest.Functions;
using Picture = FarPoint.Win.Picture;

namespace QuantReviewTest
{
    public partial class MainForm : Form
    {
        private DataSetContext _dataSetContext = new DataSetContext();
        public MainForm()
        {
            InitializeComponent();
            
            ChromVisualFunction.Register(spread, _dataSetContext);

            spread.AsWorkbook().WorkbookSet.CalculationEngine.CalcFeatures |= CalcFeatures.DynamicArray;


            spread_Sheet1.Cells[0, 0].Formula = "VF.CHROMTEXT({1,2,3},{1;2;3})";
            spread_Sheet1.Cells[5, 0].Formula = "VF.CHROM({1,2,3},{1;2;3})";
            spread_Sheet1.Rows[5].Height = spread_Sheet1.Columns[0].Width;
            spread_Sheet1.Rows[6].Height = spread_Sheet1.Columns[0].Width;
            spread_Sheet1.Rows[7].Height = spread_Sheet1.Columns[0].Width;

            var cell = spread_Sheet1.Cells[0, 4];
            cell.Value = "VF.CHROM in A6 should use the parameters to know how to draw, " +
                         "and spill draw over 9 cells. It is a dynamic array. Just like " +
                         "VF.CHROMTEXT in A1, but graphical.\n\n" +
                         "And menu-Export To Excel should also convert those cell to shapes.\n\n" +
                         "I hope this is possible...it looks very promising.\n\n" +
                         "Note that the current drawing is not real but fake random data.";
            cell.ColumnSpan = 4;
            cell.RowSpan = 8;
            cell.VerticalAlignment = CellVerticalAlignment.Top;

            var cellType = new FarPoint.Win.Spread.CellType.TextCellType();
            cellType.WordWrap = true;
            cellType.Multiline = true;
            cell.CellType = cellType;
        }

        private void showToolStripMenuItem_Click(object sender, EventArgs e)
        {
            designer.Show(spread);
        }

        private void tsmiExportToExcel_Click(object sender, EventArgs e)
        {
            //TODO: how to do this for dynamic array?
            //How to know in what cell VF.CHROM was drawing?
            for (var r = 0; r < 20; r++)
            {
                for (var c = 0; c < 20; c++)
                {
                    var cell = spread_Sheet1.Cells[5, 0];
                    if (!cell.Formula.StartsWith("VF.")) continue;
                    ConvertToShape(cell);
                }
            }

            void ConvertToShape(FarPoint.Win.Spread.Cell cell)
            {
                var rect = new Rectangle(0, 0, (int) cell.Column.Width, (int) cell.Row.Height);
                var bitMap = spread_Sheet1.SaveImage(cell.Row.Index, cell.Column.Index, 1, 1);
                var picture = new Picture(bitMap);
                var shape = new CustomShape
                {
                    Border = null,
                    BackgroundImage = picture,
                    Width = rect.Width,
                    Height = rect.Height,
                };
                spread_Sheet1.AddShape(shape, cell.Row.Index, cell.Column.Index, 0, 0, true);
            }

            var desktopFolder = Environment.GetFolderPath(Environment.SpecialFolder.Desktop);
            var file = Path.Combine(desktopFolder, DateTime.Now.ToString("yyyyMMdd-HHmmss") + ".xlsx");
            spread.SaveExcel(file, ExcelSaveFlags.NoFormulas | ExcelSaveFlags.UseOOXMLFormat);
            Process.Start(file);
        }
    }
}
